<?php
/**
 * Actions for Tutorly
 *
 * @author      VibeThemes
 * @category    Admin
 * @package     Initialization
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) exit;


class Tutorly_Actions{

    public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Tutorly_Actions();

        return self::$instance;
    }

    private function __construct(){
        //add_action( 'init', array($this,'register_testimonials' ));
        add_action('tutorly_footer',array($this,'footer'));
        add_action( 'wp_ajax_reset_googlewebfonts',array($this,'reset_googlewebfonts' ));

        add_action('vibebp_featured_style',array($this,'testimonial_block'),10,2);
        add_action('vibebp_carousel_styles_scripts',array($this,'testimonial_block_style'));
        add_action( 'login_enqueue_scripts',array($this, 'vibe_login_css' ));
        
        // WP Admin access
        add_action('current_screen',array($this,'wp_admin_access'));
        add_action('tutorly_post_content',array($this,'show_tags'));
        add_action( 'wp_enqueue_scripts', array($this,'remove_wp_block_library_css'), 100 );
        add_action('wp_footer',array($this,'footer_google_analytics'),99);
        add_action('vibebp_featured_style',array($this,'generic_block'),10,2);
        add_filter('vibe_appointments_profile_service_block_html',array($this,'profile_service_block_html'),10,5);
        add_action('vibebp_featured_style',array($this,'tutorly_trainer_block'),10,2);
        add_action('vibebp_featured_style',array($this,'tutorly_class_block'),10,2);
        add_action('tutorly_header_extras',array($this,'dark_light_theme_switch'));
        
    }

    function dark_light_theme_switch(){
        if(!empty(tutorly_get_option('dark_light_switcher'))){
            $default_dark_theme = tutorly_get_option('default_dark_theme');
            echo '<span class="dark_light_switch">

            <span class="light_theme '.(empty($default_dark_theme)?'':'active').'"><svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="1" stroke-linecap="round" stroke-linejoin="round" class="feather feather-sun"><circle cx="12" cy="12" r="5"></circle><line x1="12" y1="1" x2="12" y2="3"></line><line x1="12" y1="21" x2="12" y2="23"></line><line x1="4.22" y1="4.22" x2="5.64" y2="5.64"></line><line x1="18.36" y1="18.36" x2="19.78" y2="19.78"></line><line x1="1" y1="12" x2="3" y2="12"></line><line x1="21" y1="12" x2="23" y2="12"></line><line x1="4.22" y1="19.78" x2="5.64" y2="18.36"></line><line x1="18.36" y1="5.64" x2="19.78" y2="4.22"></line></svg></span>
            <span class="dark_theme '.(empty($default_dark_theme)?'active':'').'"><svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="1" stroke-linecap="round" stroke-linejoin="round" class="feather feather-moon"><path d="M21 12.79A9 9 0 1 1 11.21 3 7 7 0 0 0 21 12.79z"></path></svg></span>
            </span>';
        }
    }


    function tutorly_class_block($post,$style){
        
        if($style != 'tutorly_class' )
            return;

        

        $avatar = bp_core_fetch_avatar(array(
            'item_id' => $post->id,
            'object'  => 'group',
            'type'=>'full',
            'html'    => false
        ));
        $link = bp_get_group_permalink($post);

        $settings = vibe_appointments_get_settings();

        $average_rating = groups_get_groupmeta($post->id,'appointment_average_rating',true);

        if(empty($average_rating)){$average_rating=5;}
    
        ?>
        <div class="class_featured_block_wrapper tutorly_class_block class_<?php echo esc_attr($post->id); ?> flex flex-col" >
            <?php

            
           
            ?>
            <a href="<?php echo esc_url($link);?>" class="flex-1">
                <img src="<?php echo esc_url($avatar); ?>" alt="class image" class="class_avatar" />
            </a>
            <div class="class_info flex flex-col gap-6 border p-6 contentbg flex-1">
                <div class="flex gap-4">
                    
                    <div class="flex flex-col gap-2">
                        <a href="<?php echo esc_url($link);?>" class="flex-col flex">
                            <strong class="text-xl"><?php echo esc_html( $post->name ); ?></strong>
                         </a>
                         <span><?php echo implode(' ', array_slice(str_word_count( strip_tags($post->description),1), 0, 10)); ?></span>
                    </div>
                </div>
                <span class="flex-1">
                  <?php 
                    $avatar = bp_core_fetch_avatar(array(
                        'item_id' => $post->creator_id,
                        'object'  => 'user',
                        'type'=>'thumb',
                        'html'    => false
                    ));
                    echo '<span class="flex gap-2 align-center class_author">
                        <img src="'.$avatar.'" />
                        <span class="flex flex-col flex-2"><strong>'.bp_core_get_user_displayname($post->creator_id).'</strong>
                        '.(empty($settings['instructor_sub_field'])?'':'<span>'.xprofile_get_field_data($settings['instructor_sub_field'],$post->creator_id).'</span>').'
                        </span>
                    </span>';
                  ?>
                </span>
                <span class="flex gap-2 items-center justify-between"><?php 
                    if(!empty($settings['class_price'])){
                        $pid = groups_get_groupmeta($post->id,$settings['class_price'],true);
                        if(!empty($pid)){
                            $product = wc_get_product($pid);
                            echo '<span>'.$product->get_price_html().'<span class="per_slot">/'.esc_html__('slot','vibe-appointments').'</span></span>';
                        }
                    }

                ?>
                <span class="flex gap-2 items-center"><svg xmlns="http://www.w3.org/2000/svg" version="1.0" width="20px" height="20px" viewBox="0 0 36 36"><path fill="currentColor" d="M19.6859343,0.861782958 L24.8136328,8.05088572 C25.0669318,8.40601432 25.4299179,8.6717536 25.8489524,8.80883508 L34.592052,11.6690221 C35.6704701,12.021812 36.2532905,13.1657829 35.8938178,14.2241526 C35.8056709,14.4836775 35.6647294,14.7229267 35.4795411,14.9273903 L29.901129,21.0864353 C29.5299163,21.4962859 29.3444371,22.0366367 29.3872912,22.5833831 L30.1116131,31.8245163 C30.1987981,32.9368499 29.3506698,33.9079379 28.2172657,33.993502 C27.9437428,34.0141511 27.6687736,33.9809301 27.4085205,33.8957918 L18.6506147,31.0307612 C18.2281197,30.8925477 17.7713439,30.8925477 17.3488489,31.0307612 L8.59094317,33.8957918 C7.51252508,34.2485817 6.34688429,33.6765963 5.98741159,32.6182265 C5.90066055,32.3628116 5.86681029,32.0929542 5.88785051,31.8245163 L6.61217242,22.5833831 C6.65502653,22.0366367 6.46954737,21.4962859 6.09833466,21.0864353 L0.519922484,14.9273903 C-0.235294755,14.0935658 -0.158766688,12.8167745 0.690852706,12.0755971 C0.899189467,11.8938516 1.14297067,11.7555303 1.40741159,11.6690221 L10.1505113,8.80883508 C10.5695458,8.6717536 10.9325319,8.40601432 11.1858308,8.05088572 L16.3135293,0.861782958 C16.9654141,-0.0521682813 18.2488096,-0.274439442 19.1800736,0.365326425 C19.3769294,0.500563797 19.5481352,0.668586713 19.6859343,0.861782958 Z"></path></svg>
                    <?php
                        $rating = groups_get_groupmeta($post->id,'rating',true);
                        if(empty($rating)){$rating = 'N.A';}

                        echo $rating;
                    ?>
                 </span>
                </span>
                
            </div>
        </div>

        <?php
        add_action('wp_footer',[$this,'tutorly_class_Css']);
    }

    function tutorly_class_Css(){
        ?>
        <style>.class_featured_block_wrapper svg{color:#ffae30;}.class_featured_block_wrapper .class_author strong+span{font-size: .8em;opacity: 0.9;}.class_featured_block_wrapper .per_slot{opacity: 0.8;font-size: 0.8em;}.class_featured_block_wrapper .class_author img{width: 64px; height: 64px;border: 1px solid var(--border);border-radius: 50%;}.class_featured_block_wrapper .class_info{border-radius: 0 0 1rem 1rem;}.class_featured_block_wrapper img.class_avatar {border-radius: 1rem 1rem 0 0;}
        }
        </style>
        <?php
    }

    
    function tutorly_trainer_block($post,$style){
        
        if($style != 'tutorly_trainer' || !function_exists('bp_get_member_types'))
            return;

        $avatar = bp_core_fetch_avatar(array(
            'item_id'   => $post->id,
            'object'    => 'user',
            'type'      =>'thumb',
            'html'      => false
        ));
        $link = bp_core_get_user_domain($post->id);
        $member_type = bp_get_member_type($post->id);
        $name = bp_core_get_user_displayname($post->id);
        $types = bp_get_member_types(array(),'objects');

        $cover_url = bp_attachments_get_attachment(
            'url',
            array(
                'item_id' => $post->id
            )
        );

        if(empty($avatar)){
            $avatar = plugins_url('../assets/images/avatar.jpg',__FILE__);
        }


        
        if(empty($this->settings)){
            $this->settings=get_option(VIBE_APPOINTMENTS_OPTION);
        }
        $average_rating = get_user_meta($post->id,'appointment_average_rating',true);

        if(empty($average_rating)){$average_rating=5;}
    
        ?>
        <div class="member_featured_block_wrapper tutorly_booking_instructor_block member_<?php echo esc_attr($post->id); ?> flex flex-col" >
            <?php

            $this->app_settings = vibe_appointments_get_settings();
           
            ?>
            <a href="<?php echo esc_url($link);?>" class="flex-1">
                <img src="<?php echo esc_url($avatar); ?>" alt="user profile image" class="member_avatar" />
            </a>
            <div class="member_info flex flex-col gap-4 border rounded-lg p-6 contentbg flex-1">
                <div class="flex gap-4">
                    
                    <div class="flex flex-col gap-2">
                        <a href="<?php echo esc_url($link);?>" class="flex-col flex">
                            <strong class="text-xl"><?php echo esc_attr( $name ); ?></strong>
                        <?php 
                            
                            if(!empty($this->app_settings['instructor_sub_field'])){
                                $id= $this->app_settings['instructor_sub_field'];

                               
                                if(!empty($id)){
                                    echo '<span class="text-sm mt-2">'.xprofile_get_field_data($id,$post->id).'</span>';          
                                }
                            }
                            
                        
                        ?>
                         </a>
                         <?php echo do_shortcode('[instructor_labels instructor='.$post->id.']'); ?>
                    </div>
                </div>
                <span class="flex gap-2 items-center"><?php
                    echo sprintf(_x('From %s ','price instructor block','vibe-appointments'),'<strong class="price_html">'.do_shortcode('[instructor_price user="'.$post->id.'"]').'</strong>');
                    ?></span>
                <span class="flex-1">
                   
                </span>
                <?php 
                    if(!empty($this->settings['appointments_services'])){?>
                    <div class="instructor_card_services_wrapper">
                        <?php echo do_shortcode('[instructor_services user_id="'.$post->id.'"]'); ?>
                    </div>
                <?php } ?>
            </div>
        </div>
        <?php

        add_Action('wp_footer',array($this,'tutorly_trainer_block_Style'));
    }

    function tutorly_trainer_block_Style(){
        echo '<style>ul.service_list {display: flex;list-style: none;gap: 0.5rem;flex-wrap: wrap;}
                    ul.service_list li {list-style: none !important;border: 1px solid var(--bordeR);border-radius: 5px;overflow: hidden;}ul.service_list li > span {padding: 5px 12px;display:flex;border-left: 3px solid;}strong.price_html {
    font-size:1.4rem;color:var(--bg-primary);}.member_info{border-radius:0 0 1rem 1rem;}.tutorly_booking_instructor_block .member_avatar{border-radius:1rem 1rem 0 0;}</style>';
    }

    function profile_service_block_html($block_html,$atts,$service,$user_id,$sar){
        if(empty($block_html) && !empty($atts['block_style']) && $atts['block_style'] == 'alt'){

            $block_html .='<div class="post_featured_block_wrapper generic_post_block flex flex-col gap-4 border rounded-lg p-6 items-center">';
            $block_html .='<img src="'.$service['image'].'" />';
            $block_html .='<a href="'.get_permalink($service['id']).'"><h3 class="text-xl">'.$service['title'].'</h3></a><div class="post_description">';
            $block_html .= vibe_appointments_get_instructor_service_rating($service['id'],$user_id,'html');
            $block_html .='</div>';
            
            $block_html .='<div class="service_prices">';    
            if(!empty($service['credits'])){
                $block_html .= $sar['credits'].' '.__('Credits','vibe-appointments');
            }
            if(!empty($sar['price']) && !empty($sar['price']['priceHTML'])){
                $block_html .= $sar['price']['priceHTML'];
            }
            if(empty($service['credits']) && empty($sar['price'])){
                $block_html .='<span>'.__('Free Booking','vibe-appointments').'</span>';
            }
            $block_html .='</div></div>';   
        }
    }

    
    function generic_block($post,$style){

        if($style == 'generic_post'){
            ?>
                <div class="post_featured_block_wrapper generic_post_block flex flex-col gap-4 border rounded-lg p-6 items-center <?php echo esc_html($post->post_type).' '.$post->post_type.'_'.$post->ID; ?>" >
                    <?php if(has_post_thumbnail($post)){ ?>
                    <a href="<?php echo get_permalink($post->ID); ?>" title="<?php the_title_attribute(['post'=>$post]); ?>">
                        <?php echo get_the_post_thumbnail($post); ?>
                    </a>
                    <?php } ?>
                    <a href="<?php echo get_permalink($post->ID); ?>">
                        <h3 class="text-xl"><?php echo esc_html($post->post_title); ?></h3>
                    </a>
                    <div class="post_description">
                        <?php echo get_the_excerpt($post); ?>
                    </div>          
                </div>
                <?php
        }
    }

    function footer_google_analytics(){
      echo tutorly_get_option('google_analytics');
    }
    function remove_wp_block_library_css(){
        global $post;
        if(!empty( $post->ID)){
            //if elementor active remove WP Blocks
            $elementor_page = get_post_meta( $post->ID, '_elementor_edit_mode', true );
            if(!empty($elementor_page)){
                wp_dequeue_style( 'wp-block-library' );
                wp_dequeue_style( 'wp-block-library-theme' );
                wp_dequeue_style( 'wc-blocks-style' ); // Remove WooCommerce block CSS    
            }    
        }
    } 
    
 
    
    function vibe_login_css() {    //Copy this function to customize WP Admin login screen
        ?>
        <style type="text/css">
            <?php
            $wp_login_screen = vibe_get_option('wp_login_screen');
            echo esc_html($wp_login_screen);
            ?>
        </style>
        <?php 
    }
    function reset_googlewebfonts(){ 
        echo "reselecting..";
        $r = get_option('google_webfonts');
        if(isset($r)){
            delete_option('google_webfonts');
        }
        die();
    }

    /*============================================*/
    /*===========  WP ADMIN ACCESS    ============*/
    /*============================================*/
    function wp_admin_access(){
        if((defined( 'DOING_AJAX' ) && DOING_AJAX) || (defined('IFRAME_REQUEST') && IFRAME_REQUEST))
          return;
        
        $val = vibe_get_option('wp_admin_access');
        $cap = apply_filters('wplms_admin_access_capabilities',array(1=>array('edit_posts'),2=>array('manage_options')));
        $flag = 1;
        if(!empty($val) && !empty($cap[$val])){
          foreach($cap[$val] as $value){
              if(!current_user_can($value)){ 
                  $flag=0;
                  break;
                }
            }
        }
        if(empty($flag)){
          wp_redirect(home_url());
          exit;
        }
    }
   

    function footer(){
       
        if(!empty(vibe_get_option('footer_sidebar'))){
            $sidebar = apply_filters('tutorly_sidebar','footer');
            ?><div class="m-2 flex flex-wrap"><?php
            dynamic_sidebar($sidebar);
            ?></div><?php
        }
    }

    function testimonial_block($post,$style){
        if($style != 'testimonial_block')
            return;

        ?>
        <div class="testimonial_block_wrapper testimonial_'<?php echo esc_attr($post->id); ?>" >
            <div class="testimonial_content">
                <?php echo apply_filters('the_content',$post->post_content); ?>
            </div>
            <div class="testimonial_author">
                <?php echo get_the_post_thumbnail($post); ?>
                <strong><?php echo esc_html($post->post_title); ?></strong>
            </div>
        </div>
        <?php

    }

    function testimonial_block_style($style){
       
       if($style != 'testimonial_slide')
            return;

        add_action('wp_footer',function(){


        ?>
        <style>
            .testimonial_block_wrapper {
                margin: 1.5rem;
                display: flex;
                flex-direction: column;
                align-items: center;
                gap: 1rem;
            }
            .slide_block_wrapper.testimonial_slide_style {
                padding:1.5rem;
            }
            .slide_block_wrapper.testimonial_slide_style p{display:flex;text-align:center}
            .slide_block_wrapper.testimonial_slide_style:before {
                content: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="124" height="124"><path fill="none" d="M0 0h24v24H0z"/><path d="M4.583 17.321C3.553 16.227 3 15 3 13.011c0-3.5 2.457-6.637 6.03-8.188l.893 1.378c-3.335 1.804-3.987 4.145-4.247 5.621.537-.278 1.24-.375 1.929-.311 1.804.167 3.226 1.648 3.226 3.489a3.5 3.5 0 0 1-3.5 3.5c-1.073 0-2.099-.49-2.748-1.179zm10 0C13.553 16.227 13 15 13 13.011c0-3.5 2.457-6.637 6.03-8.188l.893 1.378c-3.335 1.804-3.987 4.145-4.247 5.621.537-.278 1.24-.375 1.929-.311 1.804.167 3.226 1.648 3.226 3.489a3.5 3.5 0 0 1-3.5 3.5c-1.073 0-2.099-.49-2.748-1.179z"/></svg>');
                font-family:vicon;
                font-size:3rem;position:absolute;left:-1rem;top:-1rem;
                z-index:-1;
                opacity:0.1;
            }
            .testimonial_block_wrapper .testimonial_content {
                background: #eee;
                border-radius: 1rem;
                padding: 1.5rem;
            }
            .testimonial_author{display: flex;align-items: center;flex-direction: column;}
            .testimonial_author img{
                width: 64px;
                height: 64px;
                border-radius: 50%;
                border: 1px solid #eee;
                overflow: hidden;
            }
        </style>
        <?php
    });
    }
    //Show tags in single posts
    function show_tags(){
        
        if(is_single() && !is_singular('product')){
            $prev_post = get_previous_post();
            $next_post = get_next_post();
            ob_start();
            the_tags('<div class="flex flex-wrap gap-4"><span class="tag">','</span><span class="tag">','</span></div>'); 
            $tags = ob_get_clean();
            if(!empty($tags)){


        ?>
        <div class="tags py-4 clear-both">
        <?php echo wp_kses_post($tags); ?>
        </div>
        <?php 
            }
            if(!empty($prev_post) && !empty($next_post)){
        ?>
        <div class="prev_next_posts_link flex justify-between w-full py-4 gap-4 flex-wrap">
                    
                <?php 
                $prev_post = get_previous_post();
                $next_post = get_next_post();
                ?>
                <div class="flex items-center flex-1 truncate">
                <?php

                if(!empty($prev_post))
                echo '<a href="'.get_permalink($prev_post->ID).'" class="prev link flex items-center gap-4 font-bold text-start break-all"><span class="vicon vicon-angle-left"></span>'.$prev_post->post_title.'</a>';
                ?>
                </div>
                <div class="flex items-center flex-1 justify-end truncate">
                <?php
                if(!empty($next_post))
                echo '<a href="'.get_permalink($next_post->ID).'" class="next link flex items-center gap-4 font-bold text-end break-all">'.$next_post->post_title.'<span class="vicon vicon-angle-right"></span></a>';
                ?> 
                </div>
        </div>
        <?php
            }
        }
    }
}

Tutorly_Actions::init();
