<?php
/**
 * Init functions for Tutorly
 *
 * @author      VibeThemes
 * @category    Admin
 * @package     Initialization
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) exit;


class Tutorly_Init{

    public static $instance;
    public  $option;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Tutorly_Init();

        return self::$instance;
    }

    private function __construct(){
        add_action( 'login_head', array($this,'login_head'), 100 );
        add_filter( 'template_include', array($this,'bp_template'), 99 );
        add_action( 'after_setup_theme', array($this,'content_width'), 0 );
        add_filter('widget_nav_menu_args', array($this,'widget_menu_args'));
        add_filter('tutorly_sidebar',array($this,'profile_layout'));
        add_filter('wplms_buddypress_registration_link',array($this,'registration_link'));
        add_filter('wplms_enable_signup',array($this,'enable_signup'));
        add_action('wp_head',[$this,'show_loggedin'],1);
    }

    
    function show_loggedin(){
        ?>
        <script> 
            
            let suser = sessionStorage.getItem('bp_user');

               
            if(suser){

                var user = JSON.parse(suser);  
               
                if(Object.keys(user).length){
                    document.addEventListener('DOMContentLoaded',function(){
                    document.querySelector('.vibebp-login').innerHTML='<a class="loggedin_user is_loading"><img src="'+user.avatar+'"><span class="vibebp_name">'+user.displayname+'</span></a>';

                    <?php 
                        if(function_exists('vibebp_get_setting') && !empty(vibebp_get_setting('bp_single_page'))){
                            echo "document.querySelector('.vibebp-login').setAttribute('href','".get_permalink(vibebp_get_setting('bp_single_page'))."');";
                        }
                    ?>
                    });
                }
            }
            
        </script>
        <?php
    }

    function enable_signup($flag){
        if(function_exists('bp_get_signup_allowed')){
            if(bp_get_signup_allowed()){return 1;}
        }
        $custom_registration_page = tutorly_get_option('custom_registration_page');
        if(!empty($custom_registration_page)){
            return 1;
        }

        return $flag;
    }

    function registration_link($link){
        $custom_registration_page = tutorly_get_option('custom_registration_page');
        if(!empty($custom_registration_page)){
            return get_permalink($custom_registration_page);
        }

        return $link;
    }

    function profile_layout($sidebar){
        if(is_singular('profile-layout')){
            return false;
        }
        return $sidebar;
    }
   
    function content_width() {
        $GLOBALS['content_width'] = apply_filters( 'tutorly_content_width', 1140 );
    }

    function disable_emojis_tinymce( $plugins ) {
      if ( is_array( $plugins ) ) {
        return array_diff( $plugins, array( 'wpemoji' ) );
      } else {
        return array();
      }
    }
    
    function bp_template($template){

        $flag = 0;
        if(defined('VIBE_APPOINTMENTS_OPTION')){
            if(empty($this->settings)){
                $this->settings=get_option(VIBE_APPOINTMENTS_OPTION);
            }
            if(!empty($this->settings['appointments_directory_page']))
                $flag = is_page($this->settings['appointments_directory_page']);    
        }
        
        if(function_exists('bp_is_members_directory') && (bp_is_members_directory() || bp_is_groups_directory() || $flag) ) {
            $new_template = locate_template( array( 'fullwidth.php' ) );
            if ( '' != $new_template ) {
                return $new_template ;
            }
        }
        return $template;
    }

    function get_option($field=null){
        if(empty($this->option)){
            $this->option = get_option(THEME_SHORT_NAME);
        }
        if(empty($field)){
            return $this->option;
        }

        if(empty($this->option[$field])){
            return false;
        }

        return $this->option[$field];
    }
    

    function login_head() {

        if ( has_custom_logo() ) :
            $image = wp_get_attachment_image_src( get_theme_mod( 'custom_logo' ), 'full' );
            ob_start();
            ?>
                .login h1 a {
                    background-image: url(<?php echo esc_url( $image[0] ); ?>);
                    -webkit-background-size: <?php echo absint( $image[1] )?>px;
                    background-size: <?php echo absint( $image[1] ) ?>px;
                    height: <?php echo absint( $image[2] ) ?>px;
                    width: <?php echo absint( $image[1] ) ?>px;
                }
            <?php
            $style = ob_get_clean();

            wp_add_inline_style('tutorly',$style);
        
        endif;
    }
     

    function widget_menu_args($args) { //$args is only argument needed to add custom walker
        if(empty($args['walker'])){
            return array_merge( $args, array(
                'walker' => new Vibe_Menu_Icon_Walker(),
           ));
        }
       return $args;
    }
    
}

Tutorly_Init::init();

if(!function_exists('vibe_get_option')){
    function vibe_get_option($field=null){
        $init = Tutorly_Init::init();
        return $init->get_option($field);
    }
}
if(!function_exists('tutorly_get_option')){
    function tutorly_get_option($field=null){
        $init = Tutorly_Init::init();
        return $init->get_option($field);
    }
}